import asyncio
from datetime import datetime
import json
import re

import arrow
import click
from grpc.aio import AioRpcError
from rich.table import Table
from rich.text import Text

from grid.cli import rich_click

# Not sure why we have this import Console
# When I import it normally from rich I have
# import issues
from grid.cli.cli.utilities import create_openapi_object, string2dict
from grid.cli.client import Grid
from grid.cli.core.callbacks import arrow_time_callback
from grid.cli.core.formatter import Formatable, print_to_console
from grid.cli.core.logs import cluster_logs, fancy_log_writer
from grid.cli.exceptions import LogLimitExceeded
from grid.openapi import (
    Externalv1Cluster,
    V1AWSClusterDriverSpec,
    V1ClusterDriver,
    V1ClusterPerformanceProfile,
    V1ClusterSpec,
    V1ClusterState,
    V1ClusterType,
    V1CreateClusterRequest,
    V1InstanceSpec,
    V1KubernetesClusterDriver,
)
from grid.openapi.api import cluster_service_api
from grid.protos.grid.v1.cluster_service_pb2_grpc import ClusterServiceStub
from grid.sdk.client import create_swagger_client


class ClusterList(Formatable):
    def __init__(self, clusters: [Externalv1Cluster]):
        self.clusters = clusters

    def as_json(self) -> str:
        return json.dumps(self.clusters)

    def as_table(self) -> Table:
        table = Table("id", "name", "type", "status", "created", show_header=True, header_style="bold green")
        phases = {
            V1ClusterState.QUEUED: Text("queued", style="bold yellow"),
            V1ClusterState.PENDING: Text("pending", style="bold yellow"),
            V1ClusterState.RUNNING: Text("running", style="bold green"),
            V1ClusterState.FAILED: Text("failed", style="bold red"),
            V1ClusterState.DELETED: Text("deleted", style="bold red"),
        }

        cluster_type_lookup = {
            V1ClusterType.BYOC: Text("byoc", style="bold yellow"),
            V1ClusterType.GLOBAL: Text("grid-cloud", style="bold green"),
        }
        for cluster in self.clusters:
            cluster: Externalv1Cluster
            status = phases[cluster.status.phase]
            if cluster.spec.desired_state == V1ClusterState.DELETED \
                    and cluster.status.phase != V1ClusterState.DELETED:
                status = Text("terminating", style="bold red")

            # this guard is necessary only until 0.3.93 releases which includes the `created_at`
            # field to the external API
            created_at = datetime.now()
            if hasattr(cluster, "created_at"):
                created_at = cluster.created_at

            table.add_row(
                cluster.id,
                cluster.name,
                cluster_type_lookup.get(cluster.spec.cluster_type, Text("unknown", style="red")),
                status,
                arrow.get(created_at).humanize() if created_at else "",
            )
        return table


@rich_click.group(invoke_without_command=True)
@click.pass_context
def clusters(ctx: click.Context) -> None:
    if ctx.invoked_subcommand is not None:
        return

    api_client = create_swagger_client()
    cluster_api = cluster_service_api.ClusterServiceApi(api_client)
    resp = cluster_api.cluster_service_list_clusters(phase_not_in=[V1ClusterState.DELETED])
    print_to_console(ctx, ClusterList(resp.clusters if resp.clusters else []))


default_instance_types = [
    "g2.8xlarge",
    "g3.16xlarge",
    "g3.4xlarge",
    "g3.8xlarge",
    "g3s.xlarge",
    "g4dn.12xlarge",
    "g4dn.16xlarge",
    "g4dn.2xlarge",
    "g4dn.4xlarge",
    "g4dn.8xlarge",
    "g4dn.metal",
    "g4dn.xlarge",
    "p2.16xlarge",
    "p2.8xlarge",
    "p2.xlarge",
    "p3.16xlarge",
    "p3.2xlarge",
    "p3.8xlarge",
    "p3dn.24xlarge",
    # "p4d.24xlarge",  # currently not supported
    "t2.large",
    "t2.medium",
    "t2.xlarge",
    "t2.2xlarge",
    "t3.large",
    "t3.medium",
    "t3.xlarge",
    "t3.2xlarge",
]


def _check_cluster_name_is_valid(_ctx, _param, value):
    pattern = r"^(?!-)[a-z0-9-]{1,63}(?<!-)$"
    if not re.match(pattern, value):
        raise click.ClickException(
            f"cluster name doesn't match regex pattern {pattern}\nIn simple words, use lowercase letters, numbers, and occasional -"
        )
    return value


@clusters.command()
@click.argument('name', type=str, callback=_check_cluster_name_is_valid)
@click.option('--external-id', 'external_id', type=str, required=True)
@click.option(
    '--role-arn', 'role_arn', type=str, required=True, help="AWS role ARN attached to`the associated resources."
)
@click.option(
    '--region',
    'region',
    type=str,
    required=False,
    default="us-east-1",
    help="AWS region which is used to host the associated resources."
)
@click.option(
    '--instance-types',
    'instance_types',
    type=str,
    required=False,
    default=",".join(default_instance_types),
    help="Instance types which you desire to support for computer jobs within the cluster."
)
@click.option(
    '--cost-savings',
    'cost_savings',
    type=bool,
    required=False,
    default=False,
    is_flag=True,
    help='using this flag ensures that the cluster is created with a profile that is optimized for '
    'cost saving, making runs cheaper but start-up times may increase',
)
@click.option(
    '--edit-before-creation',
    default=False,
    is_flag=True,
    help="Edit the created cluster spec before submitting to API server."
)
def aws(
    name: str,
    external_id: str,
    role_arn: str,
    region: str,
    instance_types: str,
    edit_before_creation: bool,
    cost_savings: bool,
) -> None:
    """Create a grid compute cluster with NAME from the provided AWS account details."""
    performance_profile = V1ClusterPerformanceProfile.DEFAULT
    if cost_savings:
        performance_profile = V1ClusterPerformanceProfile.COST_SAVING

    api_client = create_swagger_client()
    cluster_api = cluster_service_api.ClusterServiceApi(api_client)
    driver = V1ClusterDriver(
        kubernetes=V1KubernetesClusterDriver(
            aws=V1AWSClusterDriverSpec(
                region=region,
                role_arn=role_arn,
                external_id=external_id,
                instance_types=[V1InstanceSpec(name=x) for x in instance_types.split(",")]
            )
        )
    )
    spec = V1ClusterSpec(cluster_type=V1ClusterType.BYOC, performance_profile=performance_profile, driver=driver)
    body = V1CreateClusterRequest(name=name, spec=spec)

    new_body = body
    if edit_before_creation:
        after = click.edit(json.dumps(body.to_dict(), indent=4))
        if after is not None:
            new_body = create_openapi_object(string2dict(after), body)
        if new_body == body:
            click.echo("cluster unchanged")

    resp = cluster_api.cluster_service_create_cluster(body=new_body)
    click.echo(resp.to_str())


# TODO(nmiculinic): refactor this to be under `grid logs`
# Though I don't want to break existing workflows...
@clusters.command()
@click.argument('cluster_id', type=str, callback=_check_cluster_name_is_valid)
@click.option("-t", "--tail", is_flag=True, default=False, help="whether to tail log lines")
@click.option(
    "--from",
    "from_time",
    default="24 hours ago",
    help="The starting timestamp to query cluster logs from.",
    callback=arrow_time_callback
)
@click.option(
    "--to",
    "to_time",
    default="0 seconds ago",
    callback=arrow_time_callback,
    help="The end timestamp / relative time increment to query logs for. This is ignored when tailing logs."
)
@click.option("--limit", default=10000, help="The max number of log lines returned.")
@click.option(
    "--time-format",
    default="human",
    type=click.Choice(['human', 'iso8601'], case_sensitive=False),
    help="limit the number of log lines returned",
)
@click.option(
    "--max-per-query-limit",
    default=1000,
    help=(
        "Maximum number of items returns per single query. In most "
        "cases this should not be touched. Please see the `--limit` "
        "option instead."
    )
)
def logs(cluster_id: str, time_format: str, **kwargs):
    """Retrieve cluster logs from the managed cluster identified by CLUSTER_ID.

    These logs are streamed to stdout, and can either be tailed to view log
    lines as they are generated, or limited to a time range.
    """
    async def f():
        async with Grid.grpc_channel() as conn:
            await cluster_logs(
                cluster_id=cluster_id,
                cluster_service=ClusterServiceStub(conn),
                log_writer=fancy_log_writer(
                    time_format=time_format,
                    component="cluster",
                ),
                **kwargs,
            )

    try:
        asyncio.run(f())
    except AioRpcError as e:
        raise click.ClickException(f"cluster {cluster_id}: {e.details()}")
    except LogLimitExceeded:
        raise click.ClickException(
            f"=== more logs present but reached log line limit. Current limit --limit {kwargs['limit']}"
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/session_service.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import event_pb2 as grid_dot_v1_dot_event__pb2
from grid.protos.grid.v1 import session_pb2 as grid_dot_v1_dot_session__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/session_service.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x1dgrid/v1/session_service.proto\x12\x07grid.v1\x1a\x13grid/v1/event.proto\x1a\x15grid/v1/session.proto\"B\n\x14\x43reateSessionRequest\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"C\n\x15\x43reateSessionResponse\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"#\n\x11GetSessionRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"@\n\x12GetSessionResponse\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"\xa2\x02\n\x13ListSessionsRequest\x12%\n\x0e\x63ontinue_token\x18\x01 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x02 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x03 \x01(\x03R\x05limit\x12\x19\n\x08user_ids\x18\x04 \x03(\tR\x07userIds\x12\x1d\n\ncluster_id\x18\x05 \x01(\tR\tclusterId\x12\x30\n\x08phase_in\x18\x06 \x03(\x0e\x32\x15.grid.v1.SessionStateR\x07phaseIn\x12\x37\n\x0cphase_not_in\x18\x07 \x03(\x0e\x32\x15.grid.v1.SessionStateR\nphaseNotIn\"\x94\x01\n\x14ListSessionsResponse\x12*\n\x07session\x18\x01 \x03(\x0b\x32\x10.grid.v1.SessionR\x07session\x12%\n\x0e\x63ontinue_token\x18\x02 \x01(\tR\rcontinueToken\x12)\n\x10resource_version\x18\x03 \x01(\tR\x0fresourceVersion\"B\n\x14UpdateSessionRequest\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"C\n\x15UpdateSessionResponse\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"H\n\x1aUpdateSessionStatusRequest\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"I\n\x1bUpdateSessionStatusResponse\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"B\n\x14\x44\x65leteSessionRequest\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"C\n\x15\x44\x65leteSessionResponse\x12*\n\x07session\x18\x01 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session\"\xfc\x01\n\x14WatchSessionsRequest\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12\x14\n\x05limit\x18\x02 \x01(\x03R\x05limit\x12\x19\n\x08user_ids\x18\x04 \x03(\tR\x07userIds\x12\x1d\n\ncluster_id\x18\x05 \x01(\tR\tclusterId\x12\x30\n\x08phase_in\x18\x06 \x03(\x0e\x32\x15.grid.v1.SessionStateR\x07phaseIn\x12\x37\n\x0cphase_not_in\x18\x07 \x03(\x0e\x32\x15.grid.v1.SessionStateR\nphaseNotIn\"\x96\x01\n\x15WatchSessionsResponse\x12)\n\x10resource_version\x18\x01 \x01(\tR\x0fresourceVersion\x12&\n\x04type\x18\x02 \x01(\x0e\x32\x12.grid.v1.EventTypeR\x04type\x12*\n\x07session\x18\x03 \x01(\x0b\x32\x10.grid.v1.SessionR\x07session2\xd2\x04\n\x0eSessionService\x12P\n\rCreateSession\x12\x1d.grid.v1.CreateSessionRequest\x1a\x1e.grid.v1.CreateSessionResponse\"\x00\x12G\n\nGetSession\x12\x1a.grid.v1.GetSessionRequest\x1a\x1b.grid.v1.GetSessionResponse\"\x00\x12M\n\x0cListSessions\x12\x1c.grid.v1.ListSessionsRequest\x1a\x1d.grid.v1.ListSessionsResponse\"\x00\x12P\n\rUpdateSession\x12\x1d.grid.v1.UpdateSessionRequest\x1a\x1e.grid.v1.UpdateSessionResponse\"\x00\x12`\n\x13UpdateSessionStatus\x12#.grid.v1.UpdateSessionStatusRequest\x1a$.grid.v1.UpdateSessionStatusResponse\x12P\n\rDeleteSession\x12\x1d.grid.v1.DeleteSessionRequest\x1a\x1e.grid.v1.DeleteSessionResponse\"\x00\x12P\n\rWatchSessions\x12\x1d.grid.v1.WatchSessionsRequest\x1a\x1e.grid.v1.WatchSessionsResponse0\x01\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_event__pb2.DESCRIPTOR,grid_dot_v1_dot_session__pb2.DESCRIPTOR,])




_CREATESESSIONREQUEST = _descriptor.Descriptor(
  name='CreateSessionRequest',
  full_name='grid.v1.CreateSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.CreateSessionRequest.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=86,
  serialized_end=152,
)


_CREATESESSIONRESPONSE = _descriptor.Descriptor(
  name='CreateSessionResponse',
  full_name='grid.v1.CreateSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.CreateSessionResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=154,
  serialized_end=221,
)


_GETSESSIONREQUEST = _descriptor.Descriptor(
  name='GetSessionRequest',
  full_name='grid.v1.GetSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='grid.v1.GetSessionRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=223,
  serialized_end=258,
)


_GETSESSIONRESPONSE = _descriptor.Descriptor(
  name='GetSessionResponse',
  full_name='grid.v1.GetSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.GetSessionResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=260,
  serialized_end=324,
)


_LISTSESSIONSREQUEST = _descriptor.Descriptor(
  name='ListSessionsRequest',
  full_name='grid.v1.ListSessionsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='continue_token', full_name='grid.v1.ListSessionsRequest.continue_token', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='continueToken', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.ListSessionsRequest.resource_version', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='limit', full_name='grid.v1.ListSessionsRequest.limit', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='limit', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_ids', full_name='grid.v1.ListSessionsRequest.user_ids', index=3,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.ListSessionsRequest.cluster_id', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phase_in', full_name='grid.v1.ListSessionsRequest.phase_in', index=5,
      number=6, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phaseIn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phase_not_in', full_name='grid.v1.ListSessionsRequest.phase_not_in', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phaseNotIn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=617,
)


_LISTSESSIONSRESPONSE = _descriptor.Descriptor(
  name='ListSessionsResponse',
  full_name='grid.v1.ListSessionsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.ListSessionsResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='continue_token', full_name='grid.v1.ListSessionsResponse.continue_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='continueToken', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.ListSessionsResponse.resource_version', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=620,
  serialized_end=768,
)


_UPDATESESSIONREQUEST = _descriptor.Descriptor(
  name='UpdateSessionRequest',
  full_name='grid.v1.UpdateSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.UpdateSessionRequest.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=770,
  serialized_end=836,
)


_UPDATESESSIONRESPONSE = _descriptor.Descriptor(
  name='UpdateSessionResponse',
  full_name='grid.v1.UpdateSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.UpdateSessionResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=838,
  serialized_end=905,
)


_UPDATESESSIONSTATUSREQUEST = _descriptor.Descriptor(
  name='UpdateSessionStatusRequest',
  full_name='grid.v1.UpdateSessionStatusRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.UpdateSessionStatusRequest.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=907,
  serialized_end=979,
)


_UPDATESESSIONSTATUSRESPONSE = _descriptor.Descriptor(
  name='UpdateSessionStatusResponse',
  full_name='grid.v1.UpdateSessionStatusResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.UpdateSessionStatusResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=981,
  serialized_end=1054,
)


_DELETESESSIONREQUEST = _descriptor.Descriptor(
  name='DeleteSessionRequest',
  full_name='grid.v1.DeleteSessionRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.DeleteSessionRequest.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1056,
  serialized_end=1122,
)


_DELETESESSIONRESPONSE = _descriptor.Descriptor(
  name='DeleteSessionResponse',
  full_name='grid.v1.DeleteSessionResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.DeleteSessionResponse.session', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1124,
  serialized_end=1191,
)


_WATCHSESSIONSREQUEST = _descriptor.Descriptor(
  name='WatchSessionsRequest',
  full_name='grid.v1.WatchSessionsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.WatchSessionsRequest.resource_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='limit', full_name='grid.v1.WatchSessionsRequest.limit', index=1,
      number=2, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='limit', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_ids', full_name='grid.v1.WatchSessionsRequest.user_ids', index=2,
      number=4, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userIds', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.WatchSessionsRequest.cluster_id', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phase_in', full_name='grid.v1.WatchSessionsRequest.phase_in', index=4,
      number=6, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phaseIn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='phase_not_in', full_name='grid.v1.WatchSessionsRequest.phase_not_in', index=5,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phaseNotIn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1194,
  serialized_end=1446,
)


_WATCHSESSIONSRESPONSE = _descriptor.Descriptor(
  name='WatchSessionsResponse',
  full_name='grid.v1.WatchSessionsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='resource_version', full_name='grid.v1.WatchSessionsResponse.resource_version', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceVersion', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='type', full_name='grid.v1.WatchSessionsResponse.type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='type', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='session', full_name='grid.v1.WatchSessionsResponse.session', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='session', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1449,
  serialized_end=1599,
)

_CREATESESSIONREQUEST.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_CREATESESSIONRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_GETSESSIONRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_LISTSESSIONSREQUEST.fields_by_name['phase_in'].enum_type = grid_dot_v1_dot_session__pb2._SESSIONSTATE
_LISTSESSIONSREQUEST.fields_by_name['phase_not_in'].enum_type = grid_dot_v1_dot_session__pb2._SESSIONSTATE
_LISTSESSIONSRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_UPDATESESSIONREQUEST.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_UPDATESESSIONRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_UPDATESESSIONSTATUSREQUEST.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_UPDATESESSIONSTATUSRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_DELETESESSIONREQUEST.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_DELETESESSIONRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
_WATCHSESSIONSREQUEST.fields_by_name['phase_in'].enum_type = grid_dot_v1_dot_session__pb2._SESSIONSTATE
_WATCHSESSIONSREQUEST.fields_by_name['phase_not_in'].enum_type = grid_dot_v1_dot_session__pb2._SESSIONSTATE
_WATCHSESSIONSRESPONSE.fields_by_name['type'].enum_type = grid_dot_v1_dot_event__pb2._EVENTTYPE
_WATCHSESSIONSRESPONSE.fields_by_name['session'].message_type = grid_dot_v1_dot_session__pb2._SESSION
DESCRIPTOR.message_types_by_name['CreateSessionRequest'] = _CREATESESSIONREQUEST
DESCRIPTOR.message_types_by_name['CreateSessionResponse'] = _CREATESESSIONRESPONSE
DESCRIPTOR.message_types_by_name['GetSessionRequest'] = _GETSESSIONREQUEST
DESCRIPTOR.message_types_by_name['GetSessionResponse'] = _GETSESSIONRESPONSE
DESCRIPTOR.message_types_by_name['ListSessionsRequest'] = _LISTSESSIONSREQUEST
DESCRIPTOR.message_types_by_name['ListSessionsResponse'] = _LISTSESSIONSRESPONSE
DESCRIPTOR.message_types_by_name['UpdateSessionRequest'] = _UPDATESESSIONREQUEST
DESCRIPTOR.message_types_by_name['UpdateSessionResponse'] = _UPDATESESSIONRESPONSE
DESCRIPTOR.message_types_by_name['UpdateSessionStatusRequest'] = _UPDATESESSIONSTATUSREQUEST
DESCRIPTOR.message_types_by_name['UpdateSessionStatusResponse'] = _UPDATESESSIONSTATUSRESPONSE
DESCRIPTOR.message_types_by_name['DeleteSessionRequest'] = _DELETESESSIONREQUEST
DESCRIPTOR.message_types_by_name['DeleteSessionResponse'] = _DELETESESSIONRESPONSE
DESCRIPTOR.message_types_by_name['WatchSessionsRequest'] = _WATCHSESSIONSREQUEST
DESCRIPTOR.message_types_by_name['WatchSessionsResponse'] = _WATCHSESSIONSRESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

CreateSessionRequest = _reflection.GeneratedProtocolMessageType('CreateSessionRequest', (_message.Message,), {
  'DESCRIPTOR' : _CREATESESSIONREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.CreateSessionRequest)
  })
_sym_db.RegisterMessage(CreateSessionRequest)

CreateSessionResponse = _reflection.GeneratedProtocolMessageType('CreateSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _CREATESESSIONRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.CreateSessionResponse)
  })
_sym_db.RegisterMessage(CreateSessionResponse)

GetSessionRequest = _reflection.GeneratedProtocolMessageType('GetSessionRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETSESSIONREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetSessionRequest)
  })
_sym_db.RegisterMessage(GetSessionRequest)

GetSessionResponse = _reflection.GeneratedProtocolMessageType('GetSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _GETSESSIONRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.GetSessionResponse)
  })
_sym_db.RegisterMessage(GetSessionResponse)

ListSessionsRequest = _reflection.GeneratedProtocolMessageType('ListSessionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTSESSIONSREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ListSessionsRequest)
  })
_sym_db.RegisterMessage(ListSessionsRequest)

ListSessionsResponse = _reflection.GeneratedProtocolMessageType('ListSessionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTSESSIONSRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.ListSessionsResponse)
  })
_sym_db.RegisterMessage(ListSessionsResponse)

UpdateSessionRequest = _reflection.GeneratedProtocolMessageType('UpdateSessionRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESESSIONREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateSessionRequest)
  })
_sym_db.RegisterMessage(UpdateSessionRequest)

UpdateSessionResponse = _reflection.GeneratedProtocolMessageType('UpdateSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESESSIONRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateSessionResponse)
  })
_sym_db.RegisterMessage(UpdateSessionResponse)

UpdateSessionStatusRequest = _reflection.GeneratedProtocolMessageType('UpdateSessionStatusRequest', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESESSIONSTATUSREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateSessionStatusRequest)
  })
_sym_db.RegisterMessage(UpdateSessionStatusRequest)

UpdateSessionStatusResponse = _reflection.GeneratedProtocolMessageType('UpdateSessionStatusResponse', (_message.Message,), {
  'DESCRIPTOR' : _UPDATESESSIONSTATUSRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.UpdateSessionStatusResponse)
  })
_sym_db.RegisterMessage(UpdateSessionStatusResponse)

DeleteSessionRequest = _reflection.GeneratedProtocolMessageType('DeleteSessionRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETESESSIONREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DeleteSessionRequest)
  })
_sym_db.RegisterMessage(DeleteSessionRequest)

DeleteSessionResponse = _reflection.GeneratedProtocolMessageType('DeleteSessionResponse', (_message.Message,), {
  'DESCRIPTOR' : _DELETESESSIONRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DeleteSessionResponse)
  })
_sym_db.RegisterMessage(DeleteSessionResponse)

WatchSessionsRequest = _reflection.GeneratedProtocolMessageType('WatchSessionsRequest', (_message.Message,), {
  'DESCRIPTOR' : _WATCHSESSIONSREQUEST,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WatchSessionsRequest)
  })
_sym_db.RegisterMessage(WatchSessionsRequest)

WatchSessionsResponse = _reflection.GeneratedProtocolMessageType('WatchSessionsResponse', (_message.Message,), {
  'DESCRIPTOR' : _WATCHSESSIONSRESPONSE,
  '__module__' : 'grid.v1.session_service_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.WatchSessionsResponse)
  })
_sym_db.RegisterMessage(WatchSessionsResponse)


DESCRIPTOR._options = None

_SESSIONSERVICE = _descriptor.ServiceDescriptor(
  name='SessionService',
  full_name='grid.v1.SessionService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1602,
  serialized_end=2196,
  methods=[
  _descriptor.MethodDescriptor(
    name='CreateSession',
    full_name='grid.v1.SessionService.CreateSession',
    index=0,
    containing_service=None,
    input_type=_CREATESESSIONREQUEST,
    output_type=_CREATESESSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='GetSession',
    full_name='grid.v1.SessionService.GetSession',
    index=1,
    containing_service=None,
    input_type=_GETSESSIONREQUEST,
    output_type=_GETSESSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListSessions',
    full_name='grid.v1.SessionService.ListSessions',
    index=2,
    containing_service=None,
    input_type=_LISTSESSIONSREQUEST,
    output_type=_LISTSESSIONSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateSession',
    full_name='grid.v1.SessionService.UpdateSession',
    index=3,
    containing_service=None,
    input_type=_UPDATESESSIONREQUEST,
    output_type=_UPDATESESSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='UpdateSessionStatus',
    full_name='grid.v1.SessionService.UpdateSessionStatus',
    index=4,
    containing_service=None,
    input_type=_UPDATESESSIONSTATUSREQUEST,
    output_type=_UPDATESESSIONSTATUSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteSession',
    full_name='grid.v1.SessionService.DeleteSession',
    index=5,
    containing_service=None,
    input_type=_DELETESESSIONREQUEST,
    output_type=_DELETESESSIONRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WatchSessions',
    full_name='grid.v1.SessionService.WatchSessions',
    index=6,
    containing_service=None,
    input_type=_WATCHSESSIONSREQUEST,
    output_type=_WATCHSESSIONSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_SESSIONSERVICE)

DESCRIPTOR.services_by_name['SessionService'] = _SESSIONSERVICE

# @@protoc_insertion_point(module_scope)



from copy import deepcopy
import json
from typing import Any, Dict, io

import click
import yaml

DISK_SIZE_ERROR_MSG = "Invalid disk size, should be greater than 100Gb"


def validate_config(cfg: Dict[str, Any]) -> None:
    """
    Validates Grid config.

    Parameters
    ----------
    cfg: Dict[str, Any]
        Dictionary representing a Grid config
    """
    disk_size = cfg['compute']['train'].get('disk_size')
    if disk_size is not None and disk_size < 100:
        raise click.ClickException(DISK_SIZE_ERROR_MSG)


def read_config(value: io.TextIO):
    """
    Parameters
    ----------
    value:
        A TextIO object that has `.read()` defined for it
    """
    grid_config = value
    if grid_config:
        #  Loads the YML file as passed by the
        #  user.
        try:
            grid_config = yaml.safe_load(value.read())
            if not isinstance(grid_config, dict):
                raise Exception("Unexpected file structure")
        except Exception as e:
            raise click.BadParameter(f'Could not load your YAML config file: {e}')

        #  Adds required structure to the base
        #  YML file, if that structure isn't there.
        if 'compute' not in grid_config:
            grid_config['compute'] = {}
        if 'train' not in grid_config['compute']:
            grid_config['compute']['train'] = {}
    return grid_config


def read_config_callback(ctx, param, value):
    """
    Click callback that reads value from the config file and fix
    the structure if it doesn't exist.
    """
    return read_config(value)


def validate_disk_size_callback(ctx, param, value: int) -> int:
    """
    Validates the disk size upon user input.

    Parameters
    ----------
    ctx
        Click context
    param
        Click parameter
    value: int

    Returns
    --------
    value: int
        Unmodified value if valid
    """
    if value < 100:
        raise click.BadParameter(DISK_SIZE_ERROR_MSG)

    return value


def _duplicate_checker(js):
    result = {}
    for name, value in js:
        if name in result:
            raise ValueError('Failed to load JSON: duplicate key {0}.'.format(name))
        result[name] = value
    return result


def string2dict(text):
    if not isinstance(text, str):
        text = text.decode('utf-8')
    try:
        js = json.loads(text, object_pairs_hook=_duplicate_checker)
        return js
    except ValueError as e:
        raise ValueError('Failed to load JSON: {0}.'.format(str(e)))


def is_openapi(obj):
    return hasattr(obj, "swagger_types")


def create_openapi_object(json_obj: Dict, target: Any):
    """ Create the openAPI object from the given json dict and based on the target object
    We use the target object to make new object from the given json spec and hence target
    must be a valid object.
    """
    if not isinstance(json_obj, dict):
        raise TypeError("json_obj must be a dictionary")
    if not is_openapi(target):
        raise TypeError("target must be an openapi object")

    target_attribs = {}
    for key, value in json_obj.items():
        try:
            # user provided key is not a valid key on openapi object
            sub_target = getattr(target, key)
        except AttributeError:
            raise ValueError(f"Field {key} not found in the target object")

        if is_openapi(sub_target):  # it's an openapi object
            target_attribs[key] = create_openapi_object(value, sub_target)
        else:
            target_attribs[key] = value

        # TODO(sherin) - specifically process list and dict and do the validation. Also do the
        #  verification for enum types

    new_target = target.__class__(**target_attribs)
    return new_target



import click

from grid.cli import rich_click
from grid.cli.client import Grid
import grid.cli.globals as env
from grid.cli.utilities import is_experiment


@rich_click.command()
@rich_click.argument('run_or_experiment', type=str, nargs=1, required=True)
def view(run_or_experiment: str):
    """Grid view opens a web UI page details the output of some RUN_OR_EXPERIMENTS."""
    if is_experiment(run_or_experiment):
        view_experiment(experiment_name=run_or_experiment)
    else:
        view_run(run_name=run_or_experiment)


def view_experiment(experiment_name: str) -> None:
    client = Grid()
    experiment_id = client.get_experiment_id(experiment_name=experiment_name)
    experiment_details = client.experiment_details(experiment_id=experiment_id)
    run_name = experiment_details['getExperimentDetails']['run']['name']

    url = env.GRID_URL.replace('/graphql', '#')
    launch_url = '/'.join([url, f'runs?runName={run_name}&expId={experiment_id}'])

    click.echo()
    click.echo(f'Opening URL: {launch_url}')

    click.launch(launch_url)


def view_run(run_name: str) -> None:
    url = env.GRID_URL.replace('/graphql', '#')
    launch_url = '/'.join([url, 'view', 'run', run_name])

    click.echo()
    click.echo(f'Opening URL: {launch_url}')

    click.launch(launch_url)



from grid.cli.core.artifact import Artifact
from grid.cli.core.experiment import Experiment
from grid.cli.core.run import Run
from grid.cli.core.team import Team

__all__ = ["Experiment", "Run", "Artifact", "Team"]



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class V1SessionStatus(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'destroyed_timestamp': 'datetime',
        'jupyter_lab_token': 'str',
        'jupyter_lab_url': 'str',
        'last_state_status_transition_timestamp': 'datetime',
        'phase': 'V1SessionState',
        'ssh_url': 'str',
        'start_timestamp': 'datetime',
        'stop_timestamp': 'datetime',
        'total_run_time_seconds': 'str'
    }

    attribute_map = {
        'destroyed_timestamp': 'destroyedTimestamp',
        'jupyter_lab_token': 'jupyterLabToken',
        'jupyter_lab_url': 'jupyterLabUrl',
        'last_state_status_transition_timestamp': 'lastStateStatusTransitionTimestamp',
        'phase': 'phase',
        'ssh_url': 'sshUrl',
        'start_timestamp': 'startTimestamp',
        'stop_timestamp': 'stopTimestamp',
        'total_run_time_seconds': 'totalRunTimeSeconds'
    }

    def __init__(self, destroyed_timestamp=None, jupyter_lab_token=None, jupyter_lab_url=None, last_state_status_transition_timestamp=None, phase=None, ssh_url=None, start_timestamp=None, stop_timestamp=None, total_run_time_seconds=None, _configuration=None):  # noqa: E501
        """V1SessionStatus - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._destroyed_timestamp = None
        self._jupyter_lab_token = None
        self._jupyter_lab_url = None
        self._last_state_status_transition_timestamp = None
        self._phase = None
        self._ssh_url = None
        self._start_timestamp = None
        self._stop_timestamp = None
        self._total_run_time_seconds = None
        self.discriminator = None

        if destroyed_timestamp is not None:
            self.destroyed_timestamp = destroyed_timestamp
        if jupyter_lab_token is not None:
            self.jupyter_lab_token = jupyter_lab_token
        if jupyter_lab_url is not None:
            self.jupyter_lab_url = jupyter_lab_url
        if last_state_status_transition_timestamp is not None:
            self.last_state_status_transition_timestamp = last_state_status_transition_timestamp
        if phase is not None:
            self.phase = phase
        if ssh_url is not None:
            self.ssh_url = ssh_url
        if start_timestamp is not None:
            self.start_timestamp = start_timestamp
        if stop_timestamp is not None:
            self.stop_timestamp = stop_timestamp
        if total_run_time_seconds is not None:
            self.total_run_time_seconds = total_run_time_seconds

    @property
    def destroyed_timestamp(self):
        """Gets the destroyed_timestamp of this V1SessionStatus.  # noqa: E501


        :return: The destroyed_timestamp of this V1SessionStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._destroyed_timestamp

    @destroyed_timestamp.setter
    def destroyed_timestamp(self, destroyed_timestamp):
        """Sets the destroyed_timestamp of this V1SessionStatus.


        :param destroyed_timestamp: The destroyed_timestamp of this V1SessionStatus.  # noqa: E501
        :type: datetime
        """

        self._destroyed_timestamp = destroyed_timestamp

    @property
    def jupyter_lab_token(self):
        """Gets the jupyter_lab_token of this V1SessionStatus.  # noqa: E501


        :return: The jupyter_lab_token of this V1SessionStatus.  # noqa: E501
        :rtype: str
        """
        return self._jupyter_lab_token

    @jupyter_lab_token.setter
    def jupyter_lab_token(self, jupyter_lab_token):
        """Sets the jupyter_lab_token of this V1SessionStatus.


        :param jupyter_lab_token: The jupyter_lab_token of this V1SessionStatus.  # noqa: E501
        :type: str
        """

        self._jupyter_lab_token = jupyter_lab_token

    @property
    def jupyter_lab_url(self):
        """Gets the jupyter_lab_url of this V1SessionStatus.  # noqa: E501


        :return: The jupyter_lab_url of this V1SessionStatus.  # noqa: E501
        :rtype: str
        """
        return self._jupyter_lab_url

    @jupyter_lab_url.setter
    def jupyter_lab_url(self, jupyter_lab_url):
        """Sets the jupyter_lab_url of this V1SessionStatus.


        :param jupyter_lab_url: The jupyter_lab_url of this V1SessionStatus.  # noqa: E501
        :type: str
        """

        self._jupyter_lab_url = jupyter_lab_url

    @property
    def last_state_status_transition_timestamp(self):
        """Gets the last_state_status_transition_timestamp of this V1SessionStatus.  # noqa: E501


        :return: The last_state_status_transition_timestamp of this V1SessionStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._last_state_status_transition_timestamp

    @last_state_status_transition_timestamp.setter
    def last_state_status_transition_timestamp(self, last_state_status_transition_timestamp):
        """Sets the last_state_status_transition_timestamp of this V1SessionStatus.


        :param last_state_status_transition_timestamp: The last_state_status_transition_timestamp of this V1SessionStatus.  # noqa: E501
        :type: datetime
        """

        self._last_state_status_transition_timestamp = last_state_status_transition_timestamp

    @property
    def phase(self):
        """Gets the phase of this V1SessionStatus.  # noqa: E501


        :return: The phase of this V1SessionStatus.  # noqa: E501
        :rtype: V1SessionState
        """
        return self._phase

    @phase.setter
    def phase(self, phase):
        """Sets the phase of this V1SessionStatus.


        :param phase: The phase of this V1SessionStatus.  # noqa: E501
        :type: V1SessionState
        """

        self._phase = phase

    @property
    def ssh_url(self):
        """Gets the ssh_url of this V1SessionStatus.  # noqa: E501


        :return: The ssh_url of this V1SessionStatus.  # noqa: E501
        :rtype: str
        """
        return self._ssh_url

    @ssh_url.setter
    def ssh_url(self, ssh_url):
        """Sets the ssh_url of this V1SessionStatus.


        :param ssh_url: The ssh_url of this V1SessionStatus.  # noqa: E501
        :type: str
        """

        self._ssh_url = ssh_url

    @property
    def start_timestamp(self):
        """Gets the start_timestamp of this V1SessionStatus.  # noqa: E501


        :return: The start_timestamp of this V1SessionStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._start_timestamp

    @start_timestamp.setter
    def start_timestamp(self, start_timestamp):
        """Sets the start_timestamp of this V1SessionStatus.


        :param start_timestamp: The start_timestamp of this V1SessionStatus.  # noqa: E501
        :type: datetime
        """

        self._start_timestamp = start_timestamp

    @property
    def stop_timestamp(self):
        """Gets the stop_timestamp of this V1SessionStatus.  # noqa: E501


        :return: The stop_timestamp of this V1SessionStatus.  # noqa: E501
        :rtype: datetime
        """
        return self._stop_timestamp

    @stop_timestamp.setter
    def stop_timestamp(self, stop_timestamp):
        """Sets the stop_timestamp of this V1SessionStatus.


        :param stop_timestamp: The stop_timestamp of this V1SessionStatus.  # noqa: E501
        :type: datetime
        """

        self._stop_timestamp = stop_timestamp

    @property
    def total_run_time_seconds(self):
        """Gets the total_run_time_seconds of this V1SessionStatus.  # noqa: E501


        :return: The total_run_time_seconds of this V1SessionStatus.  # noqa: E501
        :rtype: str
        """
        return self._total_run_time_seconds

    @total_run_time_seconds.setter
    def total_run_time_seconds(self, total_run_time_seconds):
        """Sets the total_run_time_seconds of this V1SessionStatus.


        :param total_run_time_seconds: The total_run_time_seconds of this V1SessionStatus.  # noqa: E501
        :type: str
        """

        self._total_run_time_seconds = total_run_time_seconds

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(V1SessionStatus, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1SessionStatus):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1SessionStatus):
            return True

        return self.to_dict() != other.to_dict()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/event.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from grid.protos.grid.v1 import whoami_pb2 as grid_dot_v1_dot_whoami__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/event.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x13grid/v1/event.proto\x12\x07grid.v1\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\x1a google/protobuf/descriptor.proto\x1a\x14grid/v1/whoami.proto\"\x9d\x03\n\x05\x45vent\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12\x1f\n\x0bresource_id\x18\x03 \x01(\tR\nresourceId\x12#\n\rresource_name\x18\x04 \x01(\tR\x0cresourceName\x12:\n\rresource_type\x18\x05 \x01(\x0e\x32\x15.grid.v1.ResourceTypeR\x0cresourceType\x12\x17\n\x07user_id\x18\x06 \x01(\tR\x06userId\x12\x1d\n\ncluster_id\x18\x07 \x01(\tR\tclusterId\x12\x31\n\nevent_type\x18\x08 \x01(\x0e\x32\x12.grid.v1.EventTypeR\teventType\x12/\n\x06\x63\x61ller\x18\t \x01(\x0b\x32\x17.grid.v1.WhoAmIResponseR\x06\x63\x61ller\x12,\n\x06object\x18\n \x01(\x0b\x32\x14.google.protobuf.AnyR\x06object*\xa7\x01\n\tEventType\x12+\n\x16\x45VENT_TYPE_UNSPECIFIED\x10\x00\x1a\x0f\x8a\xb5\x18\x0bunspecified\x12#\n\x12\x45VENT_TYPE_CREATED\x10\x01\x1a\x0b\x8a\xb5\x18\x07\x63reated\x12#\n\x12\x45VENT_TYPE_UPDATED\x10\x02\x1a\x0b\x8a\xb5\x18\x07updated\x12#\n\x12\x45VENT_TYPE_DELETED\x10\x03\x1a\x0b\x8a\xb5\x18\x07\x64\x65leted*\x98\x02\n\x0cResourceType\x12.\n\x19RESOURCE_TYPE_UNSPECIFIED\x10\x00\x1a\x0f\x92\xb5\x18\x0bunspecified\x12,\n\x18RESOURCE_TYPE_EXPERIMENT\x10\x01\x1a\x0e\x92\xb5\x18\nexperiment\x12&\n\x15RESOURCE_TYPE_SESSION\x10\x02\x1a\x0b\x92\xb5\x18\x07session\x12*\n\x17RESOURCE_TYPE_DATASTORE\x10\x03\x1a\r\x92\xb5\x18\tdatastore\x12.\n\x19RESOURCE_TYPE_TENSORBOARD\x10\x04\x1a\x0f\x92\xb5\x18\x0btensorboard\x12&\n\x15RESOURCE_TYPE_CLUSTER\x10\x05\x1a\x0b\x92\xb5\x18\x07\x63luster:V\n\x15\x65vent_type_human_name\x12!.google.protobuf.EnumValueOptions\x18\xd1\x86\x03 \x01(\tR\x12\x65ventTypeHumanName:\\\n\x18resource_type_human_name\x12!.google.protobuf.EnumValueOptions\x18\xd2\x86\x03 \x01(\tR\x15resourceTypeHumanNameB2Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_descriptor__pb2.DESCRIPTOR,grid_dot_v1_dot_whoami__pb2.DESCRIPTOR,])

_EVENTTYPE = _descriptor.EnumDescriptor(
  name='EventType',
  full_name='grid.v1.EventType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='EVENT_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=b'\212\265\030\013unspecified',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_TYPE_CREATED', index=1, number=1,
      serialized_options=b'\212\265\030\007created',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_TYPE_UPDATED', index=2, number=2,
      serialized_options=b'\212\265\030\007updated',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='EVENT_TYPE_DELETED', index=3, number=3,
      serialized_options=b'\212\265\030\007deleted',
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=565,
  serialized_end=732,
)
_sym_db.RegisterEnumDescriptor(_EVENTTYPE)

EventType = enum_type_wrapper.EnumTypeWrapper(_EVENTTYPE)
_RESOURCETYPE = _descriptor.EnumDescriptor(
  name='ResourceType',
  full_name='grid.v1.ResourceType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=b'\222\265\030\013unspecified',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_EXPERIMENT', index=1, number=1,
      serialized_options=b'\222\265\030\nexperiment',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_SESSION', index=2, number=2,
      serialized_options=b'\222\265\030\007session',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_DATASTORE', index=3, number=3,
      serialized_options=b'\222\265\030\tdatastore',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_TENSORBOARD', index=4, number=4,
      serialized_options=b'\222\265\030\013tensorboard',
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='RESOURCE_TYPE_CLUSTER', index=5, number=5,
      serialized_options=b'\222\265\030\007cluster',
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=735,
  serialized_end=1015,
)
_sym_db.RegisterEnumDescriptor(_RESOURCETYPE)

ResourceType = enum_type_wrapper.EnumTypeWrapper(_RESOURCETYPE)
EVENT_TYPE_UNSPECIFIED = 0
EVENT_TYPE_CREATED = 1
EVENT_TYPE_UPDATED = 2
EVENT_TYPE_DELETED = 3
RESOURCE_TYPE_UNSPECIFIED = 0
RESOURCE_TYPE_EXPERIMENT = 1
RESOURCE_TYPE_SESSION = 2
RESOURCE_TYPE_DATASTORE = 3
RESOURCE_TYPE_TENSORBOARD = 4
RESOURCE_TYPE_CLUSTER = 5

EVENT_TYPE_HUMAN_NAME_FIELD_NUMBER = 50001
event_type_human_name = _descriptor.FieldDescriptor(
  name='event_type_human_name', full_name='grid.v1.event_type_human_name', index=0,
  number=50001, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=b"".decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, json_name='eventTypeHumanName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key)
RESOURCE_TYPE_HUMAN_NAME_FIELD_NUMBER = 50002
resource_type_human_name = _descriptor.FieldDescriptor(
  name='resource_type_human_name', full_name='grid.v1.resource_type_human_name', index=1,
  number=50002, type=9, cpp_type=9, label=1,
  has_default_value=False, default_value=b"".decode('utf-8'),
  message_type=None, enum_type=None, containing_type=None,
  is_extension=True, extension_scope=None,
  serialized_options=None, json_name='resourceTypeHumanName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key)


_EVENT = _descriptor.Descriptor(
  name='Event',
  full_name='grid.v1.Event',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='grid.v1.Event.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='id', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='grid.v1.Event.timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='timestamp', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_id', full_name='grid.v1.Event.resource_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_name', full_name='grid.v1.Event.resource_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='resource_type', full_name='grid.v1.Event.resource_type', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='resourceType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.Event.user_id', index=5,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.Event.cluster_id', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='event_type', full_name='grid.v1.Event.event_type', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='eventType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='caller', full_name='grid.v1.Event.caller', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='caller', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object', full_name='grid.v1.Event.object', index=9,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='object', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=149,
  serialized_end=562,
)

_EVENT.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EVENT.fields_by_name['resource_type'].enum_type = _RESOURCETYPE
_EVENT.fields_by_name['event_type'].enum_type = _EVENTTYPE
_EVENT.fields_by_name['caller'].message_type = grid_dot_v1_dot_whoami__pb2._WHOAMIRESPONSE
_EVENT.fields_by_name['object'].message_type = google_dot_protobuf_dot_any__pb2._ANY
DESCRIPTOR.message_types_by_name['Event'] = _EVENT
DESCRIPTOR.enum_types_by_name['EventType'] = _EVENTTYPE
DESCRIPTOR.enum_types_by_name['ResourceType'] = _RESOURCETYPE
DESCRIPTOR.extensions_by_name['event_type_human_name'] = event_type_human_name
DESCRIPTOR.extensions_by_name['resource_type_human_name'] = resource_type_human_name
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Event = _reflection.GeneratedProtocolMessageType('Event', (_message.Message,), {
  'DESCRIPTOR' : _EVENT,
  '__module__' : 'grid.v1.event_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.Event)
  })
_sym_db.RegisterMessage(Event)

google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(event_type_human_name)
google_dot_protobuf_dot_descriptor__pb2.EnumValueOptions.RegisterExtension(resource_type_human_name)

DESCRIPTOR._options = None
_EVENTTYPE.values_by_name["EVENT_TYPE_UNSPECIFIED"]._options = None
_EVENTTYPE.values_by_name["EVENT_TYPE_CREATED"]._options = None
_EVENTTYPE.values_by_name["EVENT_TYPE_UPDATED"]._options = None
_EVENTTYPE.values_by_name["EVENT_TYPE_DELETED"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_UNSPECIFIED"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_EXPERIMENT"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_SESSION"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_DATASTORE"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_TENSORBOARD"]._options = None
_RESOURCETYPE.values_by_name["RESOURCE_TYPE_CLUSTER"]._options = None
# @@protoc_insertion_point(module_scope)



from dataclasses import dataclass
from pathlib import Path
from typing import List, Optional

import requests
from tqdm import tqdm


@dataclass
class DownloadableObject:
    """Object that can be downloaded from an URL into a local path."""
    url: str
    download_path: str
    filename: str
    downloaded: bool = False


class Downloader:
    """
    Downloads multiple files into their respective location
    using background threads.

    Attributes
    ----------
    workers: int
        Number of background threads to use.
    progress: Progress
        Instance of a Progress bar object.
    Parameters
    ----------
    urls: List[DownloadableObject]
        List of URL objects containing.
    """
    def __init__(self, downloadable_objects: List[DownloadableObject], base_dir: str):

        self.downloadable_objects = downloadable_objects
        self.download_chunk_size = 1024  # in bytes
        self.base_dir = base_dir

    @staticmethod
    def create_dir_tree(dest_dir: str, base_dir: Optional[str] = None) -> None:
        """
        Creates directory structure for downloading file.

        Parameters
        ----------
        dest_dir: str
            Destination directory for where to download file.
        base_dir: str
            Base directory to place all targer directories into.
        """
        if base_dir:
            P = Path(base_dir) / Path(dest_dir)
        else:
            P = Path(dest_dir)

        P.mkdir(parents=True, exist_ok=True)

    def download(self):
        """
        Download multuple files to the given directory. This will download
        files using a ThreadPoolExecutor so that multiple files can
        be downloaded concurrently. We can set the concurrency level
        by changing the class' workers` attribute.
        """
        tqdm_iter = tqdm(self.downloadable_objects, unit="files")
        for obj in tqdm_iter:
            obj: DownloadableObject
            tqdm_iter.set_description(desc=obj.filename)
            # Creates directory tree if it doesn't exist
            Downloader.create_dir_tree(obj.download_path, self.base_dir)

            # Creates destination path in tree
            destination_path = Path(self.base_dir) / Path(obj.download_path) / Path(obj.filename)

            # Download file to path
            response = requests.get(obj.url, allow_redirects=True, stream=True)
            with open(destination_path, 'wb') as file:
                for chunk in response.iter_content(chunk_size=self.download_chunk_size):
                    if chunk:
                        file.write(chunk)

            # Mark object as downloaded
            obj.downloaded = True



from collections import OrderedDict
import json
from pathlib import Path
import shlex
import sys
from typing import Any, Dict, List, Optional

import click

from grid.cli import rich_click
from grid.cli.cli.utilities import read_config_callback, validate_config
from grid.cli.client import Grid
import grid.cli.globals as env
from grid.cli.types import WorkflowType
from grid.cli.utilities import check_description_isnt_too_long
from grid.sdk.action import validate_and_configure_datastore
from grid.sdk.source_code.dependency_manager import find_package_manager
from grid.sdk.utils.name_generator import unique_name


def _get_flag_args_for_grid_run() -> List[str]:
    return ['--', '--ignore_warnings', '--localdir', '--use_spot']


def _check_run_name_is_valid(_ctx, _param, value):
    """Click callback that checks if a Run contains reserved names."""
    if value is not None:
        fail = False

        #  Check if the input is alphanumeric.
        _run_name = value.replace('-', '')
        if not _run_name.isalnum():
            fail = True

        #  Check if the allowed `-` character is not used
        #  at the end of the string.
        elif value.endswith('-') or value.startswith('-'):
            fail = True

        #  Check that the run name does not contain any
        #  uppercase characters.
        elif any(x.isupper() for x in value):
            fail = True

        if fail:
            raise click.BadParameter(
                f"""

            Invalid Run name: {value} the Run name must be lower case
            alphanumeric characters or '-', start with an alphabetic
            character, and end with an alphanumeric character (e.g. 'my-name',
            or 'abc-123').

                """
            )

    return value


def _check_ignore_warnings_flag(ctx, _param, value):
    """
    Click callback that assigns the value of the ignore warnings callback
    to a global variable.
    """
    if value is not None:
        env.IGNORE_WARNINGS = value

    return value


def _generate_default_grid_config(
    instance_type: str,
    cluster: Optional[str] = None,
    use_spot: Optional[bool] = None,
    grid_strategy: Optional[str] = None,
    grid_trials: Optional[int] = None,
    gpus: Optional[int] = None,
    memory: str = None,
    cpus: Optional[int] = None,
    datastore_name: Optional[str] = None,
    datastore_version: Optional[int] = None,
    datastore_mount_dir: Optional[str] = None,
    framework: Optional[str] = None,
    localdir: Optional[bool] = None,
    dockerfile: Optional[bool] = None,
    scratch_size: Optional[int] = None,
    scratch_mount_path: Optional[str] = None,
    dependency_file: Optional[str] = None,
    resumable: Optional[bool] = None,
):
    """
    Generates a new default config file for user if user hasn't
    submitted one.
    """
    #  Relevant defaults for user.
    #  TODO: Pull from user profile when we have
    #  that information collected.
    provider = {
        'vendor': "aws",
        'credentials': "n/a",
        'region': 'us-east-1',
    }
    if cluster:
        provider['cluster'] = cluster

    _grid_config = {
        'compute': {
            'provider': provider,
            'train': {
                'framework': framework,
                'datastore_name': datastore_name,
                'datastore_version': datastore_version,
                'datastore_mount_dir': datastore_mount_dir,
                'instance': instance_type,
                'use_spot': use_spot or False,
                'nodes': 0,  # How many nodes are created at first.
                'gpus': gpus,
                'memory': memory,
                'cpus': cpus,
                'scratch': [],
                'localdir': localdir,
                'dockerfile': dockerfile,
                'resumable': bool(resumable),
            }
        }
    }

    if dependency_file:
        package_manager = find_package_manager(dependency_file)
        if not localdir:
            dependency_file = Grid.add_git_root_path(dependency_file)
        _grid_config['compute']['train']['dependency_file_info'] = {
            'package_manager': package_manager,
            'path': dependency_file
        }

    if scratch_size:
        _grid_config['compute']['train']['scratch'].append({'size_gb': scratch_size, 'mount_path': scratch_mount_path})

    #  We add a `hyper_params` key to the default config
    #  if the user has passed the `grid_strategy` argument.
    if grid_strategy:
        if not _grid_config.get('hyper_params'):
            _grid_config['hyper_params'] = {}

        _grid_config['hyper_params'] = {'settings': {'strategy': grid_strategy}, 'params': {}}

        #  We also add the key trials if that has been
        #  passed. trials is only valid when doing a `random_search`
        #  at the moment. Our parser will not be bothered by its
        #  presence if we pass it by default.
        if grid_trials:
            _grid_config['hyper_params']['settings']['trials'] = grid_trials

    if env.DEBUG:
        click.echo('Grid Config used:')
        click.echo(_grid_config)

    return _grid_config


def _check_is_valid_extension(ctx, _param, value):
    """Click callback that checks if a file is a Python script."""
    if value is None or not value.exists():
        raise click.BadParameter(f"Script must be a file.")

    if value is not None:
        valid_file_types = ['.py', '.sh', '.jl']
        if not any(value.name.endswith(e) for e in valid_file_types):
            support_email = click.style('support@grid.ai', fg='green')
            message = (
                f'We only support Python (.py), shell (.sh), or Julia (.jl) scripts. Got: {value}.'
                f'Email {support_email} if you want to request support for this file type.'
            )
            raise click.BadParameter(message)

        ctx.params['entrypoint'] = value
        return value
    return None


def _aws_node_to_nickname():
    aws_node_to_nicknames = OrderedDict({
        # AWS
        'p3.16xlarge': '8_v100_16gb',
        'p3dn.24xlarge': '8_v100_32gb',
        'g4dn.metal': '8_t4_16gb',
        'p2.8xlarge': '8_k80_12gb',
        'p3.8xlarge': '4_v100_16gb',
        'g4dn.12xlarge': '4_t4_16gb',
        'g3.16xlarge': '4_m60_8gb',
        'g3.8xlarge': '2_m60_8gb',
        'p3.2xlarge': '1_v100_16gb',
        # 'p4d.24xlarge': '8_a100_40gb',  # currently not supported
        'g4dn.8xlarge': '1_t4_16gb',
        'g4dn.4xlarge': '1_t4_16gb',
        'g4dn.2xlarge': '1_t4_16gb',
        'g4dn.xlarge': '1_t4_16gb',
        'g4dn.16xlarge': '1_t4_16gb',
        'p2.xlarge': '1_k80_12gb',
        'g3s.xlarge': '1_m60_8gb',
        'g3.4xlarge': '1_m60_8gb',
        't2.large': '2_cpu_8gb',
        't2.medium': '2_cpu_4gb'
    })
    return aws_node_to_nicknames


def _nickname_to_aws_nodes():
    aws_node_to_nickname = _aws_node_to_nickname()
    aws_nickname_to_node = {v: k for k, v in aws_node_to_nickname.items()}
    return aws_nickname_to_node


def _resolve_instance_type_nickname(ctx, _param, value):
    """
    Enables instance type shortcuts like:
    2_cpu_4gb for t2.large
    """
    nickname = value.lower()

    aws_nickname_to_node = _nickname_to_aws_nodes()

    # validate potential options for the node name
    possible_values = list(aws_nickname_to_node.keys()) + list(aws_nickname_to_node.values())
    if nickname not in possible_values:
        possible_options = '\n'.join(list(aws_nickname_to_node.keys()))
        click.BadParameter(f'{nickname} is not an available instance_type\n try one of these:\n{possible_options}')

    instance_type = nickname

    # if the value has _ then it's a nickname
    if '_' in nickname:
        instance_type = aws_nickname_to_node[nickname]
    return instance_type


def _get_instance_types(ctx, args, incomplete):
    # TODO: these should be retrieved from backend
    return list(_aws_node_to_nickname().keys())


def _parse_script_args(args):
    """Small state machine to parse the script name and script args __before__ the click parser is called."""
    flags = _get_flag_args_for_grid_run()

    # states: 'parameter_name', 'parameter_value', 'flag', 'script_name', 'script_arg'
    state = None

    click_args = []
    script_name = None
    script_args = []
    for arg in args:
        arg = str(arg)
        if state in ['script_name', 'script_arg']:
            script_args.append(arg)
        else:
            if arg.startswith('--'):
                if arg in flags:
                    state = 'flag'
                else:
                    state = 'parameter_name'
            elif state == 'parameter_name':
                state = 'parameter_value'
            else:
                state = 'script_name'
                script_name = arg

            if state != 'script_name':
                click_args.append(arg)
    return click_args, script_name, script_args


def _encode_script_args(args):
    """Parse the script args and encode them as JSON. Prepend the JSON as a '--script_args_json' parameter."""
    args, file, script_args = _parse_script_args(args)
    if file is not None:
        args = ['--script_args_json'] + [f'{json.dumps(script_args)}'] + args + [file]
    return args


class _NestedCommand(click.Command):
    def parse_args(self, ctx, args):
        args = _encode_script_args(args)
        return super().parse_args(ctx, args)


@rich_click.command(
    cls=rich_click.deprecate_grid_options(_NestedCommand), context_settings=dict(ignore_unknown_options=True)
)
@click.option(
    '--config',
    'config',
    type=click.File('r'),
    required=False,
    callback=read_config_callback,
    help='Path to Grid config YML.'
)
@click.option('--name', 'name', type=str, required=False, help='Name for this run', callback=_check_run_name_is_valid)
@click.option(
    '--description',
    'description',
    type=str,
    required=False,
    help='Run description; useful for note-keeping',
    callback=check_description_isnt_too_long
)
@click.option('--cluster', 'cluster', type=str, required=False)
@click.option(
    '--strategy',
    'strategy',
    type=click.Choice(['grid_search', 'random_search'], case_sensitive=False),
    required=False,
    help='Hyperparameter search strategy'
)
@click.option('--trials', 'trials', type=int, required=False, help='Number of trials to run hyper parameter search')
@click.option(
    '--instance_type',
    'instance_type',
    type=str,
    default='t2.medium',
    help='Instance type to start training session in',
    autocompletion=_get_instance_types,
    callback=_resolve_instance_type_nickname
)
@click.option('--gpus', 'gpus', type=int, required=False, default=0, help='Number of GPUs to allocate per experiment')
@click.option(
    '--cpus',
    'cpus',
    type=int,
    required=False,
    default=0,  # Default will be set to 1 after alerting user
    help='Number of CPUs to allocate per experiment'
)
@click.option('--memory', 'memory', type=str, required=False, help='How much memory an experiment needs')
@click.option(
    '--datastore_name', 'datastore_name', type=str, required=False, help='Datastore name to be mounted in training'
)
@click.option(
    '--datastore_version',
    'datastore_version',
    type=str,
    required=False,
    help='Datastore version to be mounted in training'
)
@click.option(
    '--datastore_mount_dir',
    'datastore_mount_dir',
    type=str,
    required=False,
    help='Directory to mount Datastore in training job'
)
@click.option(
    '--framework',
    'framework',
    type=click.Choice(['lightning', 'torch', 'tensorflow', 'julia', 'torchelastic'], case_sensitive=False),
    required=False,
    default='lightning',
    help='Framework to use in training'
)
@click.option(
    '--use_spot',
    'use_spot',
    is_flag=True,
    required=False,
    help='Use spot instance. The spot instances, or preemptive instance can be shut down at will'
)
@click.option(
    '--ignore_warnings',
    is_flag=True,
    required=False,
    help='If we should ignore warning when executing commands',
    callback=_check_ignore_warnings_flag
)
@click.option(
    '--script_args_json',
    type=str,
    required=False,
    hidden=True,
)
@click.option(
    '--scratch_size', type=int, required=False, help='The size in GB of the scratch space attached to the experiment'
)
@click.option(
    '--scratch_mount_path',
    type=str,
    required=False,
    help='The mount path to mount the scratch space',
    default='/tmp/scratch',
)
@click.option(
    '-l',
    '--localdir',
    is_flag=True,
    required=False,
)
@click.option('-d', '--dockerfile', type=str, required=False, help="Dockerfile for the image building")
@click.option(
    '--dependency_file',
    type=str,
    required=False,
    help="Dependency file path. If not provided and if either "
    "`requirements.txt` or `environment.yml` is present "
    "in the current-working-directory, that'll be used"
)
@rich_click.argument('script', type=Path, callback=_check_is_valid_extension, help='The name of the script to run.')
@rich_click.argument('script_args', nargs=-1, type=click.UNPROCESSED, help='Arguments to be passed to the script.')
# Enable this flag once the feature is ready
# @click.option(
#     '--resumable',
#     is_flag=True,
#     required=False,
#     help="Mark this experiment as resumable. It'll be rescheduled "
#     "if underlying node/instance/VM is terminated, with all "
#     "artifacts restores from the last known state."
# )
@click.pass_context
def run(
    ctx,
    script: str,
    entrypoint: Path,
    config: Optional[Dict],
    name: Optional[str],
    cluster: Optional[str],
    strategy: Optional[str],
    trials: Optional[int],
    instance_type: str,
    gpus: Optional[int],
    description: Optional[str],
    ignore_warnings: bool,
    script_args_json: Optional[str],
    script_args: Optional[Any],
    memory: Optional[str],
    cpus: Optional[int],
    datastore_name: Optional[str],
    datastore_version: Optional[str],
    datastore_mount_dir: Optional[str],
    framework: Optional[str],
    use_spot: Optional[bool],
    scratch_size: Optional[int],
    scratch_mount_path: Optional[str],
    localdir: Optional[bool],
    dockerfile: Optional[str],
    dependency_file: Optional[str],
    resumable: Optional[bool] = None,
) -> None:
    """Launch a Run from some SCRIPT with the provided SCRIPT_ARGS.

    A run is a collection of experiments which run with a single set of SCRIPT_ARGS. The
    SCRIPT_ARGS passed to the run command can represent fixed values, or a set of values
    to be searched over for each option. If a set of values are passed, a sweep (grid-search
    or random-search) will be performed, launching the desired number of experiments in
    parallel - each with a unique set of input arguments.

    The script runs on the specified instance type and Grid collects the generated
    artifacts, metrics, and logs; making them available for you to view in real time
    (or later if so desired) on either our Web UI or via this CLI.
    """
    # Captures all other user arguments that are not
    # parsed automatically.
    script_arguments = list(script_args)
    if script_args_json is not None:
        script_arguments += json.loads(script_args_json)

    # Captures invocation command exactly as it was
    # typed in the terminal. pipe.quote maintains original
    # quotation used in the CLI. We'll replace absolute
    # grid executable path with the `grid` alias.
    sys.argv[0] = "grid"
    invocation_command = " ".join(map(shlex.quote, sys.argv))

    if env.DEBUG:
        click.echo(f"Script Arguments {script_arguments}")
        click.echo(f"Entrypoint Script: {entrypoint}")
        click.echo(f"Hyperparams Search Strategy: {strategy}")
        click.echo(f"GPUs Requested: {gpus}")
        click.echo(f"Spot Requested: {use_spot}")

    client = Grid()
    client.check_is_blocked()

    # make a fun random name when user does not pass in a name
    if name is None:
        name = unique_name()

    #  If the user has not passed a grid config file,
    #  then generate one with a set of default options.

    if not config:
        _grid_config = _generate_default_grid_config(
            cluster=cluster,
            grid_strategy=strategy,
            grid_trials=trials,
            instance_type=instance_type,
            gpus=gpus,
            memory=memory,
            cpus=cpus,
            datastore_name=datastore_name,
            datastore_version=datastore_version,
            datastore_mount_dir=datastore_mount_dir,
            framework=framework,
            use_spot=use_spot,
            scratch_size=scratch_size,
            scratch_mount_path=scratch_mount_path,
            localdir=localdir,
            dockerfile=dockerfile,
            dependency_file=dependency_file,
            resumable=resumable,
        )

    else:
        _grid_config = config
        # if config file is passed but overwriting `localdir`
        if not _grid_config.get('compute'):
            raise click.BadParameter("Invalid config: missing `compute` key.")
        if not _grid_config['compute'].get('train'):
            raise click.BadParameter("Invalid config: missing `train` key.")

        if _grid_config['compute']['train'].get('localdir') is None:
            _grid_config['compute']['train']['localdir'] = localdir
        if _grid_config['compute']['train'].get('dockerfile') is None:
            _grid_config['compute']['train']['dockerfile'] = dockerfile

    validate_config(cfg=_grid_config)
    validate_and_configure_datastore(grid_config=_grid_config)

    # Warn user if they have not specified a cpu or gpu number so cpus will be set to 1
    cpus = _grid_config['compute']['train'].get('cpus')
    gpus = _grid_config['compute']['train'].get('gpus')
    if not cpus and not gpus:
        _grid_config['compute']['train']['cpus'] = 1

    #  Send to client.
    client.train(
        config=_grid_config,
        kind=WorkflowType.SCRIPT,
        run_name=name,
        run_description=description,
        entrypoint=entrypoint,
        script_args=script_arguments,
        invocation_command=invocation_command,
        localdir=localdir,
        dependency_file=dependency_file,
    )



# coding: utf-8

"""
    external/v1/external_session_service.proto

    No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)  # noqa: E501

    OpenAPI spec version: version not set
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""


import pprint
import re  # noqa: F401

import six

from grid.openapi.configuration import Configuration


class Externalv1ImageSpec(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """

    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'dependency_file_info': 'V1DependencyFileInfo',
        'dockerfile': 'str',
        'framework': 'str'
    }

    attribute_map = {
        'dependency_file_info': 'dependencyFileInfo',
        'dockerfile': 'dockerfile',
        'framework': 'framework'
    }

    def __init__(self, dependency_file_info=None, dockerfile=None, framework=None, _configuration=None):  # noqa: E501
        """Externalv1ImageSpec - a model defined in Swagger"""  # noqa: E501
        if _configuration is None:
            _configuration = Configuration()
        self._configuration = _configuration

        self._dependency_file_info = None
        self._dockerfile = None
        self._framework = None
        self.discriminator = None

        if dependency_file_info is not None:
            self.dependency_file_info = dependency_file_info
        if dockerfile is not None:
            self.dockerfile = dockerfile
        if framework is not None:
            self.framework = framework

    @property
    def dependency_file_info(self):
        """Gets the dependency_file_info of this Externalv1ImageSpec.  # noqa: E501


        :return: The dependency_file_info of this Externalv1ImageSpec.  # noqa: E501
        :rtype: V1DependencyFileInfo
        """
        return self._dependency_file_info

    @dependency_file_info.setter
    def dependency_file_info(self, dependency_file_info):
        """Sets the dependency_file_info of this Externalv1ImageSpec.


        :param dependency_file_info: The dependency_file_info of this Externalv1ImageSpec.  # noqa: E501
        :type: V1DependencyFileInfo
        """

        self._dependency_file_info = dependency_file_info

    @property
    def dockerfile(self):
        """Gets the dockerfile of this Externalv1ImageSpec.  # noqa: E501


        :return: The dockerfile of this Externalv1ImageSpec.  # noqa: E501
        :rtype: str
        """
        return self._dockerfile

    @dockerfile.setter
    def dockerfile(self, dockerfile):
        """Sets the dockerfile of this Externalv1ImageSpec.


        :param dockerfile: The dockerfile of this Externalv1ImageSpec.  # noqa: E501
        :type: str
        """

        self._dockerfile = dockerfile

    @property
    def framework(self):
        """Gets the framework of this Externalv1ImageSpec.  # noqa: E501

        name & tag of the base image (containing any runtime / system dependencies) which are used when building the experiment image.  # noqa: E501

        :return: The framework of this Externalv1ImageSpec.  # noqa: E501
        :rtype: str
        """
        return self._framework

    @framework.setter
    def framework(self, framework):
        """Sets the framework of this Externalv1ImageSpec.

        name & tag of the base image (containing any runtime / system dependencies) which are used when building the experiment image.  # noqa: E501

        :param framework: The framework of this Externalv1ImageSpec.  # noqa: E501
        :type: str
        """

        self._framework = framework

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Externalv1ImageSpec, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Externalv1ImageSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, Externalv1ImageSpec):
            return True

        return self.to_dict() != other.to_dict()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grid/v1/datastore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from grid.protos.grid.v1 import metadata_pb2 as grid_dot_v1_dot_metadata__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='grid/v1/datastore.proto',
  package='grid.v1',
  syntax='proto3',
  serialized_options=b'Z0github.com/gridai/grid/grid-backend/apis/grid/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x17grid/v1/datastore.proto\x12\x07grid.v1\x1a\x16grid/v1/metadata.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x19google/protobuf/any.proto\"\x98\x01\n\tDatastore\x12-\n\x08metadata\x18\x01 \x01(\x0b\x32\x11.grid.v1.MetadataR\x08metadata\x12*\n\x04spec\x18\x02 \x01(\x0b\x32\x16.grid.v1.DatastoreSpecR\x04spec\x12\x30\n\x06status\x18\x03 \x01(\x0b\x32\x18.grid.v1.DatastoreStatusR\x06status\"\x80\x03\n\rDatastoreSpec\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12\x16\n\x04name\x18\x02 \x01(\tB\x02\x18\x01R\x04name\x12\x18\n\x07version\x18\x03 \x01(\rR\x07version\x12\x30\n\x06source\x18\x04 \x01(\x0b\x32\x18.grid.v1.DatastoreSourceR\x06source\x12<\n\rdesired_state\x18\x05 \x01(\x0e\x32\x17.grid.v1.DatastoreStateR\x0c\x64\x65siredState\x12;\n\x0buploaded_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\nuploadedAt\x12\x1d\n\ncluster_id\x18\x07 \x01(\tR\tclusterId\x12\x19\n\x08size_mib\x18\x08 \x01(\rR\x07sizeMib\x12=\n\x0e\x64\x61tastore_type\x18\t \x01(\x0e\x32\x16.grid.v1.DatastoreTypeR\rdatastoreType\"/\n\x0f\x44\x61tastoreSource\x12\x12\n\x03\x64sn\x18\x01 \x01(\tH\x00R\x03\x64snB\x08\n\x06source\"\xf3\x01\n\x0f\x44\x61tastoreStatus\x12:\n\x05phase\x18\x01 \x01(\x0e\x32$.grid.v1.DatastoreOptimizationStatusR\x05phase\x12\x1d\n\nsize_bytes\x18\x02 \x01(\x04R\tsizeBytes\x12=\n\x0bsnapshot_id\x18\x03 \x01(\x0b\x32\x1c.grid.v1.DatastoreSnapshotIDR\nsnapshotId\x12#\n\rextracted_url\x18\x04 \x01(\tR\x0c\x65xtractedUrl\x12!\n\x0cmetadata_url\x18\x05 \x01(\tR\x0bmetadataUrl\"\x8a\x01\n\x13\x44\x61tastoreSnapshotID\x12\x32\n\x08\x65xternal\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00R\x08\x65xternal\x12\x39\n\x18k8s_volume_snapshot_name\x18\x02 \x01(\tH\x00R\x15k8sVolumeSnapshotNameB\x04\n\x02id\"A\n\x0e\x44\x61tastoreInput\x12\x10\n\x03\x64sn\x18\x01 \x01(\tR\x03\x64sn\x12\x1d\n\nmount_path\x18\x02 \x01(\tR\tmountPath*p\n\rDatastoreType\x12\x1e\n\x1a\x44\x41TASTORE_TYPE_UNSPECIFIED\x10\x00\x12$\n DATASTORE_TYPE_COMPRESSED_SOURCE\x10\x01\x12\x19\n\x15\x44\x41TASTORE_TYPE_FOLDER\x10\x02*\x8a\x01\n\x0e\x44\x61tastoreState\x12\x1f\n\x1b\x44\x41TASTORE_STATE_UNSPECIFIED\x10\x00\x12\x1d\n\x19\x44\x41TASTORE_STATE_UPLOADING\x10\x01\x12\x1b\n\x17\x44\x41TASTORE_STATE_CREATED\x10\x02\x12\x1b\n\x17\x44\x41TASTORE_STATE_DELETED\x10\x03*\xab\x02\n\x1b\x44\x61tastoreOptimizationStatus\x12-\n)DATASTORE_OPTIMIZATION_STATUS_UNSPECIFIED\x10\x00\x12+\n\'DATASTORE_OPTIMIZATION_STATUS_SUCCEEDED\x10\x01\x12(\n$DATASTORE_OPTIMIZATION_STATUS_FAILED\x10\x02\x12,\n(DATASTORE_OPTIMIZATION_STATUS_OPTIMIZING\x10\x03\x12)\n%DATASTORE_OPTIMIZATION_STATUS_DELETED\x10\x04\x12-\n)DATASTORE_OPTIMIZATION_STATUS_POD_PENDING\x10\x05\x42\x32Z0github.com/gridai/grid/grid-backend/apis/grid/v1b\x06proto3'
  ,
  dependencies=[grid_dot_v1_dot_metadata__pb2.DESCRIPTOR,google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,])

_DATASTORETYPE = _descriptor.EnumDescriptor(
  name='DatastoreType',
  full_name='grid.v1.DatastoreType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_TYPE_COMPRESSED_SOURCE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_TYPE_FOLDER', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1165,
  serialized_end=1277,
)
_sym_db.RegisterEnumDescriptor(_DATASTORETYPE)

DatastoreType = enum_type_wrapper.EnumTypeWrapper(_DATASTORETYPE)
_DATASTORESTATE = _descriptor.EnumDescriptor(
  name='DatastoreState',
  full_name='grid.v1.DatastoreState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_STATE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_STATE_UPLOADING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_STATE_CREATED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_STATE_DELETED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1280,
  serialized_end=1418,
)
_sym_db.RegisterEnumDescriptor(_DATASTORESTATE)

DatastoreState = enum_type_wrapper.EnumTypeWrapper(_DATASTORESTATE)
_DATASTOREOPTIMIZATIONSTATUS = _descriptor.EnumDescriptor(
  name='DatastoreOptimizationStatus',
  full_name='grid.v1.DatastoreOptimizationStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_SUCCEEDED', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_FAILED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_OPTIMIZING', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_DELETED', index=4, number=4,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DATASTORE_OPTIMIZATION_STATUS_POD_PENDING', index=5, number=5,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1421,
  serialized_end=1720,
)
_sym_db.RegisterEnumDescriptor(_DATASTOREOPTIMIZATIONSTATUS)

DatastoreOptimizationStatus = enum_type_wrapper.EnumTypeWrapper(_DATASTOREOPTIMIZATIONSTATUS)
DATASTORE_TYPE_UNSPECIFIED = 0
DATASTORE_TYPE_COMPRESSED_SOURCE = 1
DATASTORE_TYPE_FOLDER = 2
DATASTORE_STATE_UNSPECIFIED = 0
DATASTORE_STATE_UPLOADING = 1
DATASTORE_STATE_CREATED = 2
DATASTORE_STATE_DELETED = 3
DATASTORE_OPTIMIZATION_STATUS_UNSPECIFIED = 0
DATASTORE_OPTIMIZATION_STATUS_SUCCEEDED = 1
DATASTORE_OPTIMIZATION_STATUS_FAILED = 2
DATASTORE_OPTIMIZATION_STATUS_OPTIMIZING = 3
DATASTORE_OPTIMIZATION_STATUS_DELETED = 4
DATASTORE_OPTIMIZATION_STATUS_POD_PENDING = 5



_DATASTORE = _descriptor.Descriptor(
  name='Datastore',
  full_name='grid.v1.Datastore',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='metadata', full_name='grid.v1.Datastore.metadata', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadata', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='spec', full_name='grid.v1.Datastore.spec', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='spec', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='status', full_name='grid.v1.Datastore.status', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='status', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=121,
  serialized_end=273,
)


_DATASTORESPEC = _descriptor.Descriptor(
  name='DatastoreSpec',
  full_name='grid.v1.DatastoreSpec',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='user_id', full_name='grid.v1.DatastoreSpec.user_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='userId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='name', full_name='grid.v1.DatastoreSpec.name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', json_name='name', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='version', full_name='grid.v1.DatastoreSpec.version', index=2,
      number=3, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='version', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='source', full_name='grid.v1.DatastoreSpec.source', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='source', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desired_state', full_name='grid.v1.DatastoreSpec.desired_state', index=4,
      number=5, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='desiredState', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uploaded_at', full_name='grid.v1.DatastoreSpec.uploaded_at', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='uploadedAt', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='cluster_id', full_name='grid.v1.DatastoreSpec.cluster_id', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='clusterId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_mib', full_name='grid.v1.DatastoreSpec.size_mib', index=7,
      number=8, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sizeMib', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='datastore_type', full_name='grid.v1.DatastoreSpec.datastore_type', index=8,
      number=9, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='datastoreType', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=276,
  serialized_end=660,
)


_DATASTORESOURCE = _descriptor.Descriptor(
  name='DatastoreSource',
  full_name='grid.v1.DatastoreSource',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dsn', full_name='grid.v1.DatastoreSource.dsn', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dsn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='source', full_name='grid.v1.DatastoreSource.source',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=662,
  serialized_end=709,
)


_DATASTORESTATUS = _descriptor.Descriptor(
  name='DatastoreStatus',
  full_name='grid.v1.DatastoreStatus',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='phase', full_name='grid.v1.DatastoreStatus.phase', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='phase', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='size_bytes', full_name='grid.v1.DatastoreStatus.size_bytes', index=1,
      number=2, type=4, cpp_type=4, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='sizeBytes', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='snapshot_id', full_name='grid.v1.DatastoreStatus.snapshot_id', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='snapshotId', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='extracted_url', full_name='grid.v1.DatastoreStatus.extracted_url', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='extractedUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata_url', full_name='grid.v1.DatastoreStatus.metadata_url', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='metadataUrl', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=712,
  serialized_end=955,
)


_DATASTORESNAPSHOTID = _descriptor.Descriptor(
  name='DatastoreSnapshotID',
  full_name='grid.v1.DatastoreSnapshotID',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='external', full_name='grid.v1.DatastoreSnapshotID.external', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='external', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='k8s_volume_snapshot_name', full_name='grid.v1.DatastoreSnapshotID.k8s_volume_snapshot_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='k8sVolumeSnapshotName', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='id', full_name='grid.v1.DatastoreSnapshotID.id',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=958,
  serialized_end=1096,
)


_DATASTOREINPUT = _descriptor.Descriptor(
  name='DatastoreInput',
  full_name='grid.v1.DatastoreInput',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dsn', full_name='grid.v1.DatastoreInput.dsn', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='dsn', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='mount_path', full_name='grid.v1.DatastoreInput.mount_path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, json_name='mountPath', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1098,
  serialized_end=1163,
)

_DATASTORE.fields_by_name['metadata'].message_type = grid_dot_v1_dot_metadata__pb2._METADATA
_DATASTORE.fields_by_name['spec'].message_type = _DATASTORESPEC
_DATASTORE.fields_by_name['status'].message_type = _DATASTORESTATUS
_DATASTORESPEC.fields_by_name['source'].message_type = _DATASTORESOURCE
_DATASTORESPEC.fields_by_name['desired_state'].enum_type = _DATASTORESTATE
_DATASTORESPEC.fields_by_name['uploaded_at'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_DATASTORESPEC.fields_by_name['datastore_type'].enum_type = _DATASTORETYPE
_DATASTORESOURCE.oneofs_by_name['source'].fields.append(
  _DATASTORESOURCE.fields_by_name['dsn'])
_DATASTORESOURCE.fields_by_name['dsn'].containing_oneof = _DATASTORESOURCE.oneofs_by_name['source']
_DATASTORESTATUS.fields_by_name['phase'].enum_type = _DATASTOREOPTIMIZATIONSTATUS
_DATASTORESTATUS.fields_by_name['snapshot_id'].message_type = _DATASTORESNAPSHOTID
_DATASTORESNAPSHOTID.fields_by_name['external'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_DATASTORESNAPSHOTID.oneofs_by_name['id'].fields.append(
  _DATASTORESNAPSHOTID.fields_by_name['external'])
_DATASTORESNAPSHOTID.fields_by_name['external'].containing_oneof = _DATASTORESNAPSHOTID.oneofs_by_name['id']
_DATASTORESNAPSHOTID.oneofs_by_name['id'].fields.append(
  _DATASTORESNAPSHOTID.fields_by_name['k8s_volume_snapshot_name'])
_DATASTORESNAPSHOTID.fields_by_name['k8s_volume_snapshot_name'].containing_oneof = _DATASTORESNAPSHOTID.oneofs_by_name['id']
DESCRIPTOR.message_types_by_name['Datastore'] = _DATASTORE
DESCRIPTOR.message_types_by_name['DatastoreSpec'] = _DATASTORESPEC
DESCRIPTOR.message_types_by_name['DatastoreSource'] = _DATASTORESOURCE
DESCRIPTOR.message_types_by_name['DatastoreStatus'] = _DATASTORESTATUS
DESCRIPTOR.message_types_by_name['DatastoreSnapshotID'] = _DATASTORESNAPSHOTID
DESCRIPTOR.message_types_by_name['DatastoreInput'] = _DATASTOREINPUT
DESCRIPTOR.enum_types_by_name['DatastoreType'] = _DATASTORETYPE
DESCRIPTOR.enum_types_by_name['DatastoreState'] = _DATASTORESTATE
DESCRIPTOR.enum_types_by_name['DatastoreOptimizationStatus'] = _DATASTOREOPTIMIZATIONSTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Datastore = _reflection.GeneratedProtocolMessageType('Datastore', (_message.Message,), {
  'DESCRIPTOR' : _DATASTORE,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.Datastore)
  })
_sym_db.RegisterMessage(Datastore)

DatastoreSpec = _reflection.GeneratedProtocolMessageType('DatastoreSpec', (_message.Message,), {
  'DESCRIPTOR' : _DATASTORESPEC,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DatastoreSpec)
  })
_sym_db.RegisterMessage(DatastoreSpec)

DatastoreSource = _reflection.GeneratedProtocolMessageType('DatastoreSource', (_message.Message,), {
  'DESCRIPTOR' : _DATASTORESOURCE,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DatastoreSource)
  })
_sym_db.RegisterMessage(DatastoreSource)

DatastoreStatus = _reflection.GeneratedProtocolMessageType('DatastoreStatus', (_message.Message,), {
  'DESCRIPTOR' : _DATASTORESTATUS,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DatastoreStatus)
  })
_sym_db.RegisterMessage(DatastoreStatus)

DatastoreSnapshotID = _reflection.GeneratedProtocolMessageType('DatastoreSnapshotID', (_message.Message,), {
  'DESCRIPTOR' : _DATASTORESNAPSHOTID,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DatastoreSnapshotID)
  })
_sym_db.RegisterMessage(DatastoreSnapshotID)

DatastoreInput = _reflection.GeneratedProtocolMessageType('DatastoreInput', (_message.Message,), {
  'DESCRIPTOR' : _DATASTOREINPUT,
  '__module__' : 'grid.v1.datastore_pb2'
  # @@protoc_insertion_point(class_scope:grid.v1.DatastoreInput)
  })
_sym_db.RegisterMessage(DatastoreInput)


DESCRIPTOR._options = None
_DATASTORESPEC.fields_by_name['name']._options = None
# @@protoc_insertion_point(module_scope)
